@@ -35,12 +35,12 @@
 
 #include "cc-color-calibrate.h"
 
-#define CC_COLOR_CALIBRATE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), CC_TYPE_COLOR_CALIBRATE, CcColorCalibratePrivate))
-
 #define CALIBRATE_WINDOW_OPACITY 0.9
 
-struct _CcColorCalibratePrivate
+struct _CcColorCalibrate
 {
+  GObject          parent_instance;
+
   CdDevice        *device;
   CdSensorCap      device_kind;
   CdSensor        *sensor;
@@ -82,71 +82,71 @@ void
 cc_color_calibrate_set_kind (CcColorCalibrate *calibrate,
                              CdSensorCap kind)
 {
-  g_return_if_fail (CC_IS_COLOR_CALIB (calibrate));
-  calibrate->priv->device_kind = kind;
+  g_return_if_fail (CC_IS_COLOR_CALIBRATE (calibrate));
+  calibrate->device_kind = kind;
 }
 
 void
 cc_color_calibrate_set_temperature (CcColorCalibrate *calibrate,
                                     guint temperature)
 {
-  g_return_if_fail (CC_IS_COLOR_CALIB (calibrate));
+  g_return_if_fail (CC_IS_COLOR_CALIBRATE (calibrate));
   g_return_if_fail (temperature < 10000);
-  calibrate->priv->target_whitepoint = temperature;
+  calibrate->target_whitepoint = temperature;
 }
 
 void
 cc_color_calibrate_set_quality (CcColorCalibrate *calibrate,
                                 CdProfileQuality quality)
 {
-  g_return_if_fail (CC_IS_COLOR_CALIB (calibrate));
-  calibrate->priv->quality = quality;
+  g_return_if_fail (CC_IS_COLOR_CALIBRATE (calibrate));
+  calibrate->quality = quality;
 }
 
 CdProfileQuality
 cc_color_calibrate_get_quality (CcColorCalibrate *calibrate)
 {
-  g_return_val_if_fail (CC_IS_COLOR_CALIB (calibrate), 0);
-  return calibrate->priv->quality;
+  g_return_val_if_fail (CC_IS_COLOR_CALIBRATE (calibrate), 0);
+  return calibrate->quality;
 }
 
 void
 cc_color_calibrate_set_device (CcColorCalibrate *calibrate,
                                CdDevice *device)
 {
-  g_return_if_fail (CC_IS_COLOR_CALIB (calibrate));
+  g_return_if_fail (CC_IS_COLOR_CALIBRATE (calibrate));
   g_return_if_fail (CD_IS_DEVICE (device));
-  if (calibrate->priv->device != NULL)
-        g_object_unref (calibrate->priv->device);
-  calibrate->priv->device = g_object_ref (device);
+  if (calibrate->device != NULL)
+    g_object_unref (calibrate->device);
+  calibrate->device = g_object_ref (device);
 }
 
 void
 cc_color_calibrate_set_sensor (CcColorCalibrate *calibrate,
                                CdSensor *sensor)
 {
-  g_return_if_fail (CC_IS_COLOR_CALIB (calibrate));
+  g_return_if_fail (CC_IS_COLOR_CALIBRATE (calibrate));
   g_return_if_fail (CD_IS_SENSOR (sensor));
-  if (calibrate->priv->sensor != NULL)
-        g_object_unref (calibrate->priv->sensor);
-  calibrate->priv->sensor = g_object_ref (sensor);
+  if (calibrate->sensor != NULL)
+    g_object_unref (calibrate->sensor);
+  calibrate->sensor = g_object_ref (sensor);
 }
 
 void
 cc_color_calibrate_set_title (CcColorCalibrate *calibrate,
                               const gchar *title)
 {
-  g_return_if_fail (CC_IS_COLOR_CALIB (calibrate));
+  g_return_if_fail (CC_IS_COLOR_CALIBRATE (calibrate));
   g_return_if_fail (title != NULL);
-  g_free (calibrate->priv->title);
-  calibrate->priv->title = g_strdup (title);
+  g_free (calibrate->title);
+  calibrate->title = g_strdup (title);
 }
 
 CdProfile *
 cc_color_calibrate_get_profile (CcColorCalibrate *calibrate)
 {
-  g_return_val_if_fail (CC_IS_COLOR_CALIB (calibrate), NULL);
-  return calibrate->priv->profile;
+  g_return_val_if_fail (CC_IS_COLOR_CALIBRATE (calibrate), NULL);
+  return calibrate->profile;
 }
 
 static guint
@@ -169,21 +169,20 @@ cc_color_calibrate_calib_setup_screen (CcColorCalibrate *calibrate,
                                        const gchar *name,
                                        GError **error)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   gboolean ret = TRUE;
 
   /* get screen */
-  priv->x11_screen = gnome_rr_screen_new (gdk_screen_get_default (), error);
-  if (priv->x11_screen == NULL)
+  calibrate->x11_screen = gnome_rr_screen_new (gdk_screen_get_default (), error);
+  if (calibrate->x11_screen == NULL)
     {
       ret = FALSE;
       goto out;
     }
 
   /* get the output */
-  priv->output = gnome_rr_screen_get_output_by_name (priv->x11_screen,
+  calibrate->output = gnome_rr_screen_get_output_by_name (calibrate->x11_screen,
                                                      name);
-  if (priv->output == NULL)
+  if (calibrate->output == NULL)
     {
       ret = FALSE;
       g_set_error_literal (error,
@@ -194,8 +193,8 @@ cc_color_calibrate_calib_setup_screen (CcColorCalibrate *calibrate,
     }
 
   /* create a lookup table */
-  priv->gamma_size = _gnome_rr_output_get_gamma_size (priv->output);
-  if (priv->gamma_size == 0)
+  calibrate->gamma_size = _gnome_rr_output_get_gamma_size (calibrate->output);
+  if (calibrate->gamma_size == 0)
     {
       ret = FALSE;
       g_set_error_literal (error,
@@ -224,38 +223,35 @@ cc_color_calibrate_calib_set_output_gamma (CcColorCalibrate *calibrate,
                                            GPtrArray *array,
                                            GError **error)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   CdColorRGB *p1;
   CdColorRGB *p2;
   CdColorRGB result;
-  gboolean ret = TRUE;
   gdouble mix;
   GnomeRRCrtc *crtc;
-  guint16 *blue = NULL;
-  guint16 *green = NULL;
-  guint16 *red = NULL;
+  g_autofree guint16 *blue = NULL;
+  g_autofree guint16 *green = NULL;
+  g_autofree guint16 *red = NULL;
   guint i;
 
   /* no length? */
   if (array->len == 0)
     {
-      ret = FALSE;
       g_set_error_literal (error,
                            CD_SESSION_ERROR,
                            CD_SESSION_ERROR_INTERNAL,
                            "no data in the CLUT array");
-      goto out;
+      return FALSE;
     }
 
   /* convert to a type X understands of the right size */
-  red = g_new (guint16, priv->gamma_size);
-  green = g_new (guint16, priv->gamma_size);
-  blue = g_new (guint16, priv->gamma_size);
+  red = g_new (guint16, calibrate->gamma_size);
+  green = g_new (guint16, calibrate->gamma_size);
+  blue = g_new (guint16, calibrate->gamma_size);
   cd_color_rgb_set (&result, 1.0, 1.0, 1.0);
-  for (i = 0; i < priv->gamma_size; i++)
+  for (i = 0; i < calibrate->gamma_size; i++)
     {
       mix = (gdouble) (array->len - 1) /
-            (gdouble) (priv->gamma_size - 1) *
+            (gdouble) (calibrate->gamma_size - 1) *
             (gdouble) i;
       p1 = g_ptr_array_index (array, (guint) floor (mix));
       p2 = g_ptr_array_index (array, (guint) ceil (mix));
@@ -269,24 +265,19 @@ cc_color_calibrate_calib_set_output_gamma (CcColorCalibrate *calibrate,
     }
 
   /* send to LUT */
-  crtc = gnome_rr_output_get_crtc (priv->output);
+  crtc = gnome_rr_output_get_crtc (calibrate->output);
   if (crtc == NULL)
     {
-      ret = FALSE;
       g_set_error (error,
                    CD_SESSION_ERROR,
                    CD_SESSION_ERROR_INTERNAL,
                    "failed to get ctrc for %s",
-                   gnome_rr_output_get_name (priv->output));
-      goto out;
+                   gnome_rr_output_get_name (calibrate->output));
+      return FALSE;
     }
-  gnome_rr_crtc_set_gamma (crtc, priv->gamma_size,
+  gnome_rr_crtc_set_gamma (crtc, calibrate->gamma_size,
                            red, green, blue);
-out:
-  g_free (red);
-  g_free (green);
-  g_free (blue);
-  return ret;
+  return TRUE;
 }
 
 static void
@@ -295,7 +286,6 @@ cc_color_calibrate_property_changed_cb (GDBusProxy *proxy,
                                         GStrv invalidated_properties,
                                         CcColorCalibrate *calibrate)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   gboolean ret;
   GtkWidget *widget;
   guint value;
@@ -305,7 +295,7 @@ cc_color_calibrate_property_changed_cb (GDBusProxy *proxy,
                           "u", &value);
   if (ret)
     {
-      widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+      widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                    "progressbar_status"));
       gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (widget),
                                      value / 100.0f);
@@ -318,10 +308,8 @@ cc_color_calibrate_interaction_required (CcColorCalibrate *calibrate,
                                          const gchar *message,
                                          const gchar *image_path)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   const gchar *message_transl;
   gboolean show_button_start = FALSE;
-  GdkPixbuf *pixbuf;
   GtkImage *img;
   GtkLabel *label;
   GtkWidget *widget;
@@ -331,27 +319,26 @@ cc_color_calibrate_interaction_required (CcColorCalibrate *calibrate,
     image_path = "preferences-color-symbolic";
 
   /* set image */
-  img = GTK_IMAGE (gtk_builder_get_object (priv->builder,
+  img = GTK_IMAGE (gtk_builder_get_object (calibrate->builder,
                                            "image_status"));
   if (image_path != NULL && image_path[0] != '\0')
     {
+      g_autoptr(GdkPixbuf) pixbuf = NULL;
+
       g_debug ("showing image %s", image_path);
       pixbuf = gdk_pixbuf_new_from_file_at_size (image_path,
                                                  400, 400,
                                                  NULL);
       if (pixbuf != NULL)
-        {
-          gtk_image_set_from_pixbuf (img, pixbuf);
-          g_object_unref (pixbuf);
-        }
+        gtk_image_set_from_pixbuf (img, pixbuf);
       gtk_widget_set_visible (GTK_WIDGET (img), TRUE);
-      gtk_widget_set_visible (GTK_WIDGET (priv->sample_widget), FALSE);
+      gtk_widget_set_visible (GTK_WIDGET (calibrate->sample_widget), FALSE);
     }
   else
     {
       g_debug ("hiding image");
       gtk_widget_set_visible (GTK_WIDGET (img), FALSE);
-      gtk_widget_set_visible (GTK_WIDGET (priv->sample_widget), TRUE);
+      gtk_widget_set_visible (GTK_WIDGET (calibrate->sample_widget), TRUE);
     }
 
   /* set new status */
@@ -384,15 +371,15 @@ cc_color_calibrate_interaction_required (CcColorCalibrate *calibrate,
       message_transl = message;
       break;
     }
-  label = GTK_LABEL (gtk_builder_get_object (priv->builder,
+  label = GTK_LABEL (gtk_builder_get_object (calibrate->builder,
                                              "label_status"));
   gtk_label_set_label (label, message_transl);
 
   /* show the correct button */
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_start"));
   gtk_widget_set_visible (widget, show_button_start);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_resume"));
   gtk_widget_set_visible (widget, !show_button_start);
 }
@@ -440,24 +427,23 @@ cc_color_calibrate_finished (CcColorCalibrate *calibrate,
                              const gchar *error_fallback)
 {
   GtkWidget *widget;
-  GString *str;
+  g_autoptr(GString) str = NULL;
   const gchar *tmp;
-  CcColorCalibratePrivate *priv = calibrate->priv;
 
   /* save failure so we can get this after we've quit the loop */
-  calibrate->priv->session_error_code = code;
+  calibrate->session_error_code = code;
 
   /* show correct buttons */
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_cancel"));
   gtk_widget_set_visible (widget, FALSE);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_start"));
   gtk_widget_set_visible (widget, FALSE);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_resume"));
   gtk_widget_set_visible (widget, FALSE);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_done"));
   gtk_widget_set_visible (widget, TRUE);
 
@@ -483,10 +469,9 @@ cc_color_calibrate_finished (CcColorCalibrate *calibrate,
   /* TRANSLATORS: The user can now remove the sensor from the screen */
   g_string_append (str, _("You can remove the calibration device."));
 
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "label_status"));
   gtk_label_set_label (GTK_LABEL (widget), str->str);
-  g_string_free (str, TRUE);
 }
 
 static void
@@ -496,7 +481,6 @@ cc_color_calibrate_signal_cb (GDBusProxy *proxy,
                               GVariant *parameters,
                               CcColorCalibrate *calibrate)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   CdColorRGB color;
   CdColorRGB *color_tmp;
   const gchar *image = NULL;
@@ -504,12 +488,11 @@ cc_color_calibrate_signal_cb (GDBusProxy *proxy,
   const gchar *profile_path = NULL;
   const gchar *str = NULL;
   gboolean ret;
-  GError *error = NULL;
+  g_autoptr(GError) error = NULL;
   GPtrArray *array = NULL;
   GtkImage *img;
   GtkLabel *label;
-  GVariantIter *iter;
-  GVariant *dict = NULL;
+  g_autoptr(GVariant) dict = NULL;
 
   if (g_strcmp0 (signal_name, "Finished") == 0)
     {
@@ -521,9 +504,9 @@ cc_color_calibrate_signal_cb (GDBusProxy *proxy,
       g_variant_lookup (dict, "ErrorDetails", "&s", &str);
       ret = g_variant_lookup (dict, "ProfilePath", "&s", &profile_path);
       if (ret)
-        priv->profile = cd_profile_new_with_object_path (profile_path);
+        calibrate->profile = cd_profile_new_with_object_path (profile_path);
       cc_color_calibrate_finished (calibrate, error_code, str);
-      goto out;
+      return;
     }
   if (g_strcmp0 (signal_name, "UpdateSample") == 0)
     {
@@ -531,30 +514,30 @@ cc_color_calibrate_signal_cb (GDBusProxy *proxy,
                      &color.R,
                      &color.G,
                      &color.B);
-      img = GTK_IMAGE (gtk_builder_get_object (priv->builder,
+      img = GTK_IMAGE (gtk_builder_get_object (calibrate->builder,
                                                "image_status"));
       gtk_widget_set_visible (GTK_WIDGET (img), FALSE);
-      gtk_widget_set_visible (GTK_WIDGET (priv->sample_widget), TRUE);
-      cd_sample_widget_set_color (CD_SAMPLE_WIDGET (priv->sample_widget),
+      gtk_widget_set_visible (GTK_WIDGET (calibrate->sample_widget), TRUE);
+      cd_sample_widget_set_color (CD_SAMPLE_WIDGET (calibrate->sample_widget),
                                   &color);
 
       /* for Lenovo W700 and W520 laptops we almost fullscreen the
        * sample widget as the device is actually embedded in the
        * palmrest! */
-      if (cd_sensor_get_embedded (priv->sensor))
+      if (cd_sensor_get_embedded (calibrate->sensor))
         {
           g_debug ("Making sample window larger for embedded sensor");
-          gtk_widget_set_size_request (priv->sample_widget, 1000, 600);
+          gtk_widget_set_size_request (calibrate->sample_widget, 1000, 600);
         }
 
       /* set the generic label too */
-      label = GTK_LABEL (gtk_builder_get_object (priv->builder,
+      label = GTK_LABEL (gtk_builder_get_object (calibrate->builder,
                                                  "label_status"));
       /* TRANSLATORS: The user has to be careful not to knock the
        * display off the screen (although we do cope if this is
        * detected early enough) */
       gtk_label_set_label (label, _("Do not disturb the calibration device while in progress"));
-      goto out;
+      return;
     }
   if (g_strcmp0 (signal_name, "InteractionRequired") == 0)
     {
@@ -570,10 +553,12 @@ cc_color_calibrate_signal_cb (GDBusProxy *proxy,
                                                code,
                                                message,
                                                image);
-      goto out;
+      return;
     }
   if (g_strcmp0 (signal_name, "UpdateGamma") == 0)
     {
+      g_autoptr(GVariantIter) iter = NULL;
+
       g_variant_get (parameters,
                      "(a(ddd))",
                      &iter);
@@ -587,7 +572,6 @@ cc_color_calibrate_signal_cb (GDBusProxy *proxy,
           cd_color_rgb_copy (&color, color_tmp);
           g_ptr_array_add (array, color_tmp);
         }
-      g_variant_iter_free (iter);
       ret = cc_color_calibrate_calib_set_output_gamma (calibrate,
                                                        array,
                                                        &error);
@@ -595,25 +579,21 @@ cc_color_calibrate_signal_cb (GDBusProxy *proxy,
         {
           g_warning ("failed to update gamma: %s",
                      error->message);
-          g_error_free (error);
-          goto out;
+          return;
         }
-      goto out;
+      return;
     }
   g_warning ("got unknown signal %s", signal_name);
-out:
-  if (dict != NULL)
-    g_variant_unref (dict);
 }
 
 static void
 cc_color_calibrate_cancel (CcColorCalibrate *calibrate)
 {
-  GVariant *retval;
-  GError *error = NULL;
+  g_autoptr(GVariant) retval = NULL;
+  g_autoptr(GError) error = NULL;
 
   /* cancel the calibration to ensure the helper quits */
-  retval = g_dbus_proxy_call_sync (calibrate->priv->proxy_helper,
+  retval = g_dbus_proxy_call_sync (calibrate->proxy_helper,
                                    "Cancel",
                                    NULL,
                                    G_DBUS_CALL_FLAGS_NONE,
@@ -621,15 +601,10 @@ cc_color_calibrate_cancel (CcColorCalibrate *calibrate)
                                    NULL,
                                    &error);
   if (retval == NULL)
-    {
-      g_warning ("Failed to send Cancel: %s", error->message);
-      g_error_free (error);
-    }
+    g_warning ("Failed to send Cancel: %s", error->message);
 
   /* return */
-  g_main_loop_quit (calibrate->priv->loop);
-  if (retval != NULL)
-    g_variant_unref (retval);
+  g_main_loop_quit (calibrate->loop);
 }
 
 static gboolean
@@ -696,7 +671,7 @@ cc_color_calibrate_window_state_cb (GtkWidget *widget,
                                     CcColorCalibrate *calibrate)
 {
   gboolean ret;
-  GError *error = NULL;
+  g_autoptr(GError) error = NULL;
   GdkEventWindowState *event_state = (GdkEventWindowState *) event;
   GtkWindow *window = GTK_WINDOW (widget);
 
@@ -708,13 +683,10 @@ cc_color_calibrate_window_state_cb (GtkWidget *widget,
 
   /* resize to the correct screen */
   ret = cc_color_calibrate_move_and_resize_window (window,
-                                                   calibrate->priv->device,
+                                                   calibrate->device,
                                                    &error);
   if (!ret)
-    {
-      g_warning ("Failed to resize window: %s", error->message);
-      g_error_free (error);
-    }
+    g_warning ("Failed to resize window: %s", error->message);
   return TRUE;
 }
 
@@ -722,27 +694,26 @@ static void
 cc_color_calibrate_button_done_cb (GtkWidget *widget,
                                    CcColorCalibrate *calibrate)
 {
-  g_main_loop_quit (calibrate->priv->loop);
+  g_main_loop_quit (calibrate->loop);
 }
 
 static void
 cc_color_calibrate_button_start_cb (GtkWidget *widget,
                                     CcColorCalibrate *calibrate)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
-  GError *error = NULL;
-  GVariant *retval;
+  g_autoptr(GError) error = NULL;
+  g_autoptr(GVariant) retval = NULL;
 
   /* set correct buttons */
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_start"));
   gtk_widget_set_visible (widget, FALSE);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_resume"));
   gtk_widget_set_visible (widget, FALSE);
 
   /* continue */
-  retval = g_dbus_proxy_call_sync (calibrate->priv->proxy_helper,
+  retval = g_dbus_proxy_call_sync (calibrate->proxy_helper,
                                    "Resume",
                                    NULL,
                                    G_DBUS_CALL_FLAGS_NONE,
@@ -750,12 +721,7 @@ cc_color_calibrate_button_start_cb (GtkWidget *widget,
                                    NULL,
                                    &error);
   if (retval == NULL)
-    {
-      g_warning ("Failed to send Resume: %s", error->message);
-      g_error_free (error);
-    }
-  if (retval != NULL)
-    g_variant_unref (retval);
+    g_warning ("Failed to send Resume: %s", error->message);
 }
 
 static void
@@ -799,36 +765,34 @@ cc_color_calibrate_alpha_screen_changed_cb (GtkWindow *window,
 static void
 cc_color_calibrate_uninhibit (CcColorCalibrate *calibrate)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   GtkApplication *application;
 
-  if (priv->inhibit_fd != -1)
+  if (calibrate->inhibit_fd != -1)
     {
-      close (priv->inhibit_fd);
-      priv->inhibit_fd = -1;
+      close (calibrate->inhibit_fd);
+      calibrate->inhibit_fd = -1;
     }
 
-  if (priv->inhibit_cookie != 0)
+  if (calibrate->inhibit_cookie != 0)
     {
       application = GTK_APPLICATION (g_application_get_default ());
-      gtk_application_uninhibit (application, priv->inhibit_cookie);
-      priv->inhibit_cookie = 0;
+      gtk_application_uninhibit (application, calibrate->inhibit_cookie);
+      calibrate->inhibit_cookie = 0;
     }
 }
 
 static void
 cc_color_calibrate_inhibit (CcColorCalibrate *calibrate, GtkWindow *window)
 {
-  GError *error = NULL;
+  g_autoptr(GError) error = NULL;
   gint idx;
-  GUnixFDList *fd_list = NULL;
-  GVariant *retval;
+  g_autoptr(GUnixFDList) fd_list = NULL;
+  g_autoptr(GVariant) retval = NULL;
   GtkApplication *application;
-  CcColorCalibratePrivate *priv = calibrate->priv;
 
   /* inhibit basically everything we can */
   application = GTK_APPLICATION (g_application_get_default ());
-  priv->inhibit_cookie = gtk_application_inhibit (application,
+  calibrate->inhibit_cookie = gtk_application_inhibit (application,
                                                   window,
                                                   GTK_APPLICATION_INHIBIT_LOGOUT |
                                                   GTK_APPLICATION_INHIBIT_SWITCH |
@@ -837,7 +801,7 @@ cc_color_calibrate_inhibit (CcColorCalibrate *calibrate, GtkWindow *window)
                                                   "Display calibration in progress");
 
   /* tell logind to disallow the lid switch */
-  retval = g_dbus_proxy_call_with_unix_fd_list_sync (priv->proxy_inhibit,
+  retval = g_dbus_proxy_call_with_unix_fd_list_sync (calibrate->proxy_inhibit,
                                                      "Inhibit",
                                                      g_variant_new ("(ssss)",
                                                                     "shutdown:"
@@ -856,67 +820,59 @@ cc_color_calibrate_inhibit (CcColorCalibrate *calibrate, GtkWindow *window)
   if (retval == NULL)
     {
       g_warning ("Failed to send Inhibit: %s", error->message);
-      g_error_free (error);
-      goto out;
+      return;
     }
   g_variant_get (retval, "(h)", &idx);
-  priv->inhibit_fd = g_unix_fd_list_get (fd_list, idx, &error);
-  if (priv->inhibit_fd == -1)
+  calibrate->inhibit_fd = g_unix_fd_list_get (fd_list, idx, &error);
+  if (calibrate->inhibit_fd == -1)
     {
       g_warning ("Failed to receive system inhibitor fd: %s", error->message);
-      g_error_free (error);
-      goto out;
+      return;
     }
-  g_debug ("System inhibitor fd is %d", priv->inhibit_fd);
-out:
-  if (fd_list != NULL)
-    g_object_unref (fd_list);
-  if (retval != NULL)
-    g_variant_unref (retval);
+  g_debug ("System inhibitor fd is %d", calibrate->inhibit_fd);
 }
 
 gboolean
 cc_color_calibrate_setup (CcColorCalibrate *calibrate,
                           GError **error)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   gboolean ret = TRUE;
 
-  g_return_val_if_fail (CC_IS_COLOR_CALIB (calibrate), FALSE);
-  g_return_val_if_fail (calibrate->priv->device_kind != CD_SENSOR_CAP_UNKNOWN, FALSE);
+  g_return_val_if_fail (CC_IS_COLOR_CALIBRATE (calibrate), FALSE);
+  g_return_val_if_fail (calibrate->device_kind != CD_SENSOR_CAP_UNKNOWN, FALSE);
 
   /* use logind to disable system state idle */
-  priv->proxy_inhibit = cc_object_storage_create_dbus_proxy_sync (G_BUS_TYPE_SYSTEM,
-                                                                  G_DBUS_PROXY_FLAGS_NONE,
-                                                                  "org.freedesktop.login1",
-                                                                  "/org/freedesktop/login1",
-                                                                  "org.freedesktop.login1.Manager",
-                                                                  NULL,
-                                                                  error);
-  if (priv->proxy_inhibit == NULL)
+  calibrate->proxy_inhibit = cc_object_storage_create_dbus_proxy_sync (G_BUS_TYPE_SYSTEM,
+								       G_DBUS_PROXY_FLAGS_NONE,
+								       "org.freedesktop.login1",
+								       "/org/freedesktop/login1",
+								       "org.freedesktop.login1.Manager",
+								       NULL,
+								       error);
+  if (calibrate->proxy_inhibit == NULL)
     {
       ret = FALSE;
       goto out;
     }
 
   /* start the calibration session daemon */
-  priv->proxy_helper = cc_object_storage_create_dbus_proxy_sync (G_BUS_TYPE_SESSION,
-                                                                 G_DBUS_PROXY_FLAGS_NONE,
-                                                                 CD_SESSION_DBUS_SERVICE,
-                                                                 CD_SESSION_DBUS_PATH,
-                                                                 CD_SESSION_DBUS_INTERFACE_DISPLAY,
-                                                                 NULL,
-                                                                 error);
-  if (priv->proxy_helper == NULL)
+  calibrate->proxy_helper = cc_object_storage_create_dbus_proxy_sync (G_BUS_TYPE_SESSION,
+								      G_DBUS_PROXY_FLAGS_NONE,
+								      CD_SESSION_DBUS_SERVICE,
+								      CD_SESSION_DBUS_PATH,
+								      CD_SESSION_DBUS_INTERFACE_DISPLAY,
+								      NULL,
+								      error);
+  if (calibrate->proxy_helper == NULL)
     {
       ret = FALSE;
       goto out;
     }
-  g_signal_connect_object (priv->proxy_helper,
+  g_signal_connect_object (calibrate->proxy_helper,
                            "g-properties-changed",
                            G_CALLBACK (cc_color_calibrate_property_changed_cb),
                            calibrate, 0);
-  g_signal_connect_object (priv->proxy_helper,
+  g_signal_connect_object (calibrate->proxy_helper,
                            "g-signal",
                            G_CALLBACK (cc_color_calibrate_signal_cb),
                            calibrate, 0);
@@ -929,102 +885,94 @@ cc_color_calibrate_start (CcColorCalibrate *calibrate,
                           GtkWindow *parent,
                           GError **error)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
   const gchar *name;
-  gboolean ret;
   GtkWidget *widget;
   GtkWindow *window;
   GVariantBuilder builder;
-  GVariant *retval = NULL;
+  g_autoptr(GVariant) retval = NULL;
 
-  g_return_val_if_fail (CC_IS_COLOR_CALIB (calibrate), FALSE);
+  g_return_val_if_fail (CC_IS_COLOR_CALIBRATE (calibrate), FALSE);
 
   /* get screen */
-  name = cd_device_get_metadata_item (priv->device,
+  name = cd_device_get_metadata_item (calibrate->device,
                                       CD_DEVICE_METADATA_XRANDR_NAME);
-  ret = cc_color_calibrate_calib_setup_screen (calibrate, name, error);
-  if (!ret)
-    goto out;
+  if (!cc_color_calibrate_calib_setup_screen (calibrate, name, error))
+    return FALSE;
 
   g_variant_builder_init (&builder, G_VARIANT_TYPE_ARRAY);
   g_variant_builder_add (&builder,
                          "{sv}",
                          "Quality",
-                         g_variant_new_uint32 (priv->quality));
+                         g_variant_new_uint32 (calibrate->quality));
   g_variant_builder_add (&builder,
                          "{sv}",
                          "Whitepoint",
-                         g_variant_new_uint32 (priv->target_whitepoint));
+                         g_variant_new_uint32 (calibrate->target_whitepoint));
   g_variant_builder_add (&builder,
                          "{sv}",
                          "Gamma",
-                         g_variant_new_double (priv->target_gamma));
+                         g_variant_new_double (calibrate->target_gamma));
   g_variant_builder_add (&builder,
                          "{sv}",
                          "Title",
-                         g_variant_new_string (priv->title));
+                         g_variant_new_string (calibrate->title));
   g_variant_builder_add (&builder,
                          "{sv}",
                          "DeviceKind",
-                         g_variant_new_uint32 (priv->device_kind));
-  retval = g_dbus_proxy_call_sync (priv->proxy_helper,
+                         g_variant_new_uint32 (calibrate->device_kind));
+  retval = g_dbus_proxy_call_sync (calibrate->proxy_helper,
                                    "Start",
                                    g_variant_new ("(ssa{sv})",
-                                                  cd_device_get_id (priv->device),
-                                                  cd_sensor_get_id (priv->sensor),
+                                                  cd_device_get_id (calibrate->device),
+                                                  cd_sensor_get_id (calibrate->sensor),
                                                   &builder),
                                    G_DBUS_CALL_FLAGS_NONE,
                                    -1,
                                    NULL,
                                    error);
   if (retval == NULL)
-    {
-      ret = FALSE;
-      goto out;
-    }
+    return FALSE;
 
   /* set this above our parent */
-  window = GTK_WINDOW (gtk_builder_get_object (priv->builder,
+  window = GTK_WINDOW (gtk_builder_get_object (calibrate->builder,
                                                "dialog_calibrate"));
   gtk_window_set_modal (window, TRUE);
   gtk_widget_show (GTK_WIDGET (window));
 
   /* show correct buttons */
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_cancel"));
   gtk_widget_set_visible (widget, TRUE);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_start"));
   gtk_widget_set_visible (widget, TRUE);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_resume"));
   gtk_widget_set_visible (widget, FALSE);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_done"));
   gtk_widget_set_visible (widget, FALSE);
 
   /* stop the computer from auto-suspending or turning off the screen */
   cc_color_calibrate_inhibit (calibrate, parent);
 
-  g_main_loop_run (priv->loop);
+  g_main_loop_run (calibrate->loop);
   gtk_widget_hide (GTK_WIDGET (window));
 
   /* we can go idle now */
   cc_color_calibrate_uninhibit (calibrate);
 
   /* see if we failed */
-  if (calibrate->priv->session_error_code != CD_SESSION_ERROR_NONE)
+  if (calibrate->session_error_code != CD_SESSION_ERROR_NONE)
     {
-      ret = FALSE;
       g_set_error_literal (error,
                            CD_SESSION_ERROR,
                            CD_SESSION_ERROR_INTERNAL,
                            "failed to calibrate");
+      return FALSE;
     }
-out:
-  if (retval != NULL)
-    g_variant_unref (retval);
-  return ret;
+
+  return TRUE;
 }
 
 static gboolean
@@ -1041,17 +989,16 @@ static void
 cc_color_calibrate_finalize (GObject *object)
 {
   CcColorCalibrate *calibrate = CC_COLOR_CALIBRATE (object);
-  CcColorCalibratePrivate *priv = calibrate->priv;
 
-  g_clear_pointer (&priv->window, gtk_widget_destroy);
-  g_clear_object (&priv->builder);
-  g_clear_object (&priv->device);
-  g_clear_object (&priv->proxy_helper);
-  g_clear_object (&priv->proxy_inhibit);
-  g_clear_object (&priv->sensor);
-  g_clear_object (&priv->x11_screen);
-  g_free (priv->title);
-  g_main_loop_unref (priv->loop);
+  g_clear_pointer (&calibrate->window, gtk_widget_destroy);
+  g_clear_object (&calibrate->builder);
+  g_clear_object (&calibrate->device);
+  g_clear_object (&calibrate->proxy_helper);
+  g_clear_object (&calibrate->proxy_inhibit);
+  g_clear_object (&calibrate->sensor);
+  g_clear_object (&calibrate->x11_screen);
+  g_free (calibrate->title);
+  g_main_loop_unref (calibrate->loop);
 
   G_OBJECT_CLASS (cc_color_calibrate_parent_class)->finalize (object);
 }
@@ -1061,73 +1008,65 @@ cc_color_calibrate_class_init (CcColorCalibrateClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   object_class->finalize = cc_color_calibrate_finalize;
-
-  g_type_class_add_private (klass, sizeof (CcColorCalibratePrivate));
 }
 
 static void
 cc_color_calibrate_init (CcColorCalibrate *calibrate)
 {
-  CcColorCalibratePrivate *priv = calibrate->priv;
-  GError *error = NULL;
+  g_autoptr(GError) error = NULL;
   gint retval;
-  GSettings *settings;
+  g_autoptr(GSettings) settings = NULL;
   GtkBox *box;
   GtkWidget *widget;
   GtkWindow *window;
 
-  calibrate->priv = priv = CC_COLOR_CALIBRATE_GET_PRIVATE (calibrate);
-  calibrate->priv->loop = g_main_loop_new (NULL, FALSE);
-  calibrate->priv->inhibit_fd = -1;
+  calibrate->loop = g_main_loop_new (NULL, FALSE);
+  calibrate->inhibit_fd = -1;
 
   /* load UI */
-  priv->builder = gtk_builder_new ();
-  retval = gtk_builder_add_from_resource (priv->builder,
+  calibrate->builder = gtk_builder_new ();
+  retval = gtk_builder_add_from_resource (calibrate->builder,
                                           "/org/gnome/control-center/color/color-calibrate.ui",
                                           &error);
   if (retval == 0)
-    {
-      g_warning ("Could not load interface: %s", error->message);
-      g_error_free (error);
-    }
+    g_warning ("Could not load interface: %s", error->message);
 
   /* add sample widget */
-  box = GTK_BOX (gtk_builder_get_object (priv->builder,
+  box = GTK_BOX (gtk_builder_get_object (calibrate->builder,
                  "vbox_status"));
-  priv->sample_widget = cd_sample_widget_new ();
-  gtk_widget_set_size_request (priv->sample_widget, 400, 400);
-  gtk_box_pack_start (box, priv->sample_widget, FALSE, FALSE, 0);
-  gtk_box_reorder_child (box, priv->sample_widget, 0);
-  gtk_widget_set_vexpand (priv->sample_widget, FALSE);
-  gtk_widget_set_hexpand (priv->sample_widget, FALSE);
+  calibrate->sample_widget = cd_sample_widget_new ();
+  gtk_widget_set_size_request (calibrate->sample_widget, 400, 400);
+  gtk_box_pack_start (box, calibrate->sample_widget, FALSE, FALSE, 0);
+  gtk_box_reorder_child (box, calibrate->sample_widget, 0);
+  gtk_widget_set_vexpand (calibrate->sample_widget, FALSE);
+  gtk_widget_set_hexpand (calibrate->sample_widget, FALSE);
 
   /* get defaults */
   settings = g_settings_new (COLORD_SETTINGS_SCHEMA);
-  calibrate->priv->target_whitepoint = g_settings_get_int (settings, "display-whitepoint");
-  calibrate->priv->target_gamma = g_settings_get_double (settings, "display-gamma");
-  g_object_unref (settings);
+  calibrate->target_whitepoint = g_settings_get_int (settings, "display-whitepoint");
+  calibrate->target_gamma = g_settings_get_double (settings, "display-gamma");
 
   /* connect to buttons */
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_start"));
   g_signal_connect (widget, "clicked",
                     G_CALLBACK (cc_color_calibrate_button_start_cb), calibrate);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_resume"));
   g_signal_connect (widget, "clicked",
                     G_CALLBACK (cc_color_calibrate_button_start_cb), calibrate);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_done"));
   g_signal_connect (widget, "clicked",
                     G_CALLBACK (cc_color_calibrate_button_done_cb), calibrate);
-  widget = GTK_WIDGET (gtk_builder_get_object (priv->builder,
+  widget = GTK_WIDGET (gtk_builder_get_object (calibrate->builder,
                                                "button_cancel"));
   g_signal_connect (widget, "clicked",
                     G_CALLBACK (cc_color_calibrate_button_cancel_cb), calibrate);
   gtk_widget_show (widget);
 
   /* setup the specialist calibration window */
-  window = GTK_WINDOW (gtk_builder_get_object (priv->builder,
+  window = GTK_WINDOW (gtk_builder_get_object (calibrate->builder,
                                                "dialog_calibrate"));
   g_signal_connect (window, "draw",
                     G_CALLBACK (cc_color_calibrate_alpha_window_draw), calibrate);
@@ -1142,7 +1081,7 @@ cc_color_calibrate_init (CcColorCalibrate *calibrate)
   cc_color_calibrate_alpha_screen_changed_cb (GTK_WINDOW (window), NULL, calibrate);
   g_signal_connect (window, "screen-changed",
                     G_CALLBACK (cc_color_calibrate_alpha_screen_changed_cb), calibrate);
-  priv->window = window;
+  calibrate->window = window;
 }
 
 CcColorCalibrate *
