@@ -42,7 +42,6 @@ typedef struct {
         GtkListBoxRow *more_item;
         GtkWidget *filter_entry;
         GtkWidget *language_list;
-        GtkWidget *scrolledwindow;
         gboolean showing_extra;
         gchar *language;
         gchar **filter_words;
@@ -202,9 +201,9 @@ language_visible (GtkListBoxRow *row,
 {
         GtkDialog *chooser = user_data;
         CcLanguageChooserPrivate *priv = GET_PRIVATE (chooser);
-        gchar *locale_name = NULL;
-        gchar *locale_current_name = NULL;
-        gchar *locale_untranslated_name = NULL;
+        g_autofree gchar *locale_name = NULL;
+        g_autofree gchar *locale_current_name = NULL;
+        g_autofree gchar *locale_untranslated_name = NULL;
         gboolean is_extra;
         gboolean visible;
 
@@ -219,29 +218,21 @@ language_visible (GtkListBoxRow *row,
         if (!priv->filter_words)
                 return TRUE;
 
-        visible = FALSE;
-
         locale_name =
                 cc_util_normalize_casefold_and_unaccent (g_object_get_data (G_OBJECT (row), "locale-name"));
         visible = match_all (priv->filter_words, locale_name);
         if (visible)
-                goto out;
+                return TRUE;
 
         locale_current_name =
                 cc_util_normalize_casefold_and_unaccent (g_object_get_data (G_OBJECT (row), "locale-current-name"));
         visible = match_all (priv->filter_words, locale_current_name);
         if (visible)
-                goto out;
+                return TRUE;
 
         locale_untranslated_name =
                 cc_util_normalize_casefold_and_unaccent (g_object_get_data (G_OBJECT (row), "locale-untranslated-name"));
-        visible = match_all (priv->filter_words, locale_untranslated_name);
-
-out:
-        g_free (locale_untranslated_name);
-        g_free (locale_current_name);
-        g_free (locale_name);
-        return visible;
+        return match_all (priv->filter_words, locale_untranslated_name);
 }
 
 static gint
@@ -267,7 +258,7 @@ static void
 filter_changed (GtkDialog *chooser)
 {
         CcLanguageChooserPrivate *priv = GET_PRIVATE (chooser);
-        gchar *filter_contents = NULL;
+        g_autofree gchar *filter_contents = NULL;
 
         g_clear_pointer (&priv->filter_words, g_strfreev);
 
@@ -279,7 +270,6 @@ filter_changed (GtkDialog *chooser)
                 return;
         }
         priv->filter_words = g_strsplit_set (g_strstrip (filter_contents), " ", 0);
-        g_free (filter_contents);
         gtk_list_box_set_placeholder (GTK_LIST_BOX (priv->language_list), priv->no_results);
         gtk_list_box_invalidate_filter (GTK_LIST_BOX (priv->language_list));
 }
@@ -288,18 +278,12 @@ static void
 show_more (GtkDialog *chooser, gboolean visible)
 {
         CcLanguageChooserPrivate *priv = GET_PRIVATE (chooser);
-        GtkWidget *widget;
         gint width, height;
 
         gtk_window_get_size (GTK_WINDOW (chooser), &width, &height);
         gtk_widget_set_size_request (GTK_WIDGET (chooser), width, height);
         gtk_window_set_resizable (GTK_WINDOW (chooser), TRUE);
 
-        widget = priv->scrolledwindow;
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (widget),
-                                        GTK_POLICY_NEVER,
-                                        visible ? GTK_POLICY_AUTOMATIC : GTK_POLICY_NEVER);
-
         gtk_widget_set_visible (priv->filter_entry, visible);
         gtk_widget_grab_focus (visible ? priv->filter_entry : priv->language_list);
 
@@ -312,7 +296,8 @@ set_locale_id (GtkDialog *chooser,
                const gchar       *locale_id)
 {
         CcLanguageChooserPrivate *priv = GET_PRIVATE (chooser);
-        GList *children, *l;
+        g_autoptr(GList) children = NULL;
+        GList *l;
 
         children = gtk_container_get_children (GTK_CONTAINER (priv->language_list));
         for (l = children; l; l = l->next) {
@@ -337,7 +322,6 @@ set_locale_id (GtkDialog *chooser,
                         gtk_widget_set_opacity (check, 0.0);
                 }
         }
-        g_list_free (children);
 
         g_free (priv->language);
         priv->language = g_strdup (locale_id);
@@ -404,30 +388,27 @@ cc_language_chooser_private_free (gpointer data)
 GtkWidget *
 cc_language_chooser_new (GtkWidget *parent)
 {
-        GtkBuilder *builder;
+        g_autoptr(GtkBuilder) builder = NULL;
         GtkWidget *chooser;
         CcLanguageChooserPrivate *priv;
-        GError *error = NULL;
+        g_autoptr(GError) error = NULL;
 
         g_resources_register (cc_common_get_resource ());
 
         builder = gtk_builder_new ();
         if (gtk_builder_add_from_resource (builder, "/org/gnome/control-center/common/language-chooser.ui", &error) == 0) {
-                g_object_unref (builder);
                 g_warning ("failed to load language chooser: %s", error->message);
-                g_error_free (error);
                 return NULL;
         }
 
         chooser = WID ("language-dialog");
         priv = g_new0 (CcLanguageChooserPrivate, 1);
         g_object_set_data_full (G_OBJECT (chooser), "private", priv, cc_language_chooser_private_free);
-        g_object_set_data_full (G_OBJECT (chooser), "builder", builder, g_object_unref);
+        g_object_set_data_full (G_OBJECT (chooser), "builder", g_object_ref (builder), g_object_unref);
 
         priv->done_button = WID ("ok-button");
         priv->filter_entry = WID ("language-filter-entry");
         priv->language_list = WID ("language-list");
-        priv->scrolledwindow = WID ("language-scrolledwindow");
         priv->more_item = more_widget_new ();
         /* We ref-sink here so we can reuse this widget multiple times */
         priv->no_results = g_object_ref_sink (no_results_widget_new ());
