@@ -13,14 +13,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by: Matthias Clasen <mclasen@redhat.com>
  */
 
-#ifndef __CC_LANGUAGE_CHOOSER_H__
-#define __CC_LANGUAGE_CHOOSER_H__
+#pragma once
 
 #include <gtk/gtk.h>
 #include <glib-object.h>
@@ -34,5 +32,3 @@ void         cc_language_chooser_set_language (GtkWidget   *chooser,
                                                const gchar *language);
 
 G_END_DECLS
-
-#endif /* __CC_LANGUAGE_CHOOSER_H__ */
