@@ -23,8 +23,9 @@
 #include "net-device-wifi.h"
 #include "network-dialogs.h"
 
+#include "shell/cc-application.h"
+#include "shell/cc-debug.h"
 #include "shell/cc-object-storage.h"
-#include "list-box-helper.h"
 
 #include <glib/gi18n.h>
 #include <NetworkManager.h>
@@ -81,6 +82,67 @@ enum
   N_PROPS
 };
 
+/* Static init function */
+
+static void
+update_panel_visibility (NMClient *client)
+{
+  const GPtrArray *devices;
+  CcApplication *application;
+  gboolean visible;
+  guint i;
+
+  CC_TRACE_MSG ("Updating Wi-Fi panel visibility");
+
+  devices = nm_client_get_devices (client);
+  visible = FALSE;
+
+  for (i = 0; devices && i < devices->len; i++)
+    {
+      NMDevice *device = g_ptr_array_index (devices, i);
+
+      visible |= NM_IS_DEVICE_WIFI (device);
+
+      if (visible)
+        break;
+    }
+
+  /* Set the new visibility */
+  application = CC_APPLICATION (g_application_get_default ());
+  cc_shell_model_set_panel_visibility (cc_application_get_model (application),
+                                       "wifi",
+                                       visible ? CC_PANEL_VISIBLE : CC_PANEL_VISIBLE_IN_SEARCH);
+
+  g_debug ("Wi-Fi panel visible: %s", visible ? "yes" : "no");
+}
+
+void
+cc_wifi_panel_static_init_func (void)
+{
+  NMClient *client;
+
+  g_debug ("Monitoring NetworkManager for Wi-Fi devices");
+
+  /* Create and store a NMClient instance if it doesn't exist yet */
+  if (!cc_object_storage_has_object (CC_OBJECT_NMCLIENT))
+    {
+      client = nm_client_new (NULL, NULL);
+      cc_object_storage_add_object (CC_OBJECT_NMCLIENT, client);
+      g_object_unref (client);
+    }
+
+  client = cc_object_storage_get_object (CC_OBJECT_NMCLIENT);
+
+  /* Update the panel visibility and monitor for changes */
+
+  g_signal_connect (client, "device-added", G_CALLBACK (update_panel_visibility), NULL);
+  g_signal_connect (client, "device-removed", G_CALLBACK (update_panel_visibility), NULL);
+
+  update_panel_visibility (client);
+
+  g_object_unref (client);
+}
+
 /* Auxiliary methods */
 
 static void
@@ -91,7 +153,7 @@ add_wifi_device (CcWifiPanel *self,
   NetObject *net_device;
 
   /* Only manage Wi-Fi devices */
-  if (!NM_IS_DEVICE_WIFI (device))
+  if (!NM_IS_DEVICE_WIFI (device) || !nm_device_get_managed (device))
     return;
 
   /* Create the NetDevice */
@@ -373,7 +435,7 @@ device_removed_cb (NMClient    *client,
   const gchar *id;
   guint i;
 
-  if (!NM_IS_DEVICE_WIFI (device))
+  if (!NM_IS_DEVICE_WIFI (device) || !nm_device_get_managed (device))
     return;
 
   id = nm_device_get_udi (device);
