@@ -113,6 +113,22 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+static InkLevelData *
+ink_level_data_new (void)
+{
+  return g_slice_new0 (InkLevelData);
+}
+
+static void
+ink_level_data_free (InkLevelData *data)
+{
+  g_clear_pointer (&data->marker_names, g_free);
+  g_clear_pointer (&data->marker_levels, g_free);
+  g_clear_pointer (&data->marker_colors, g_free);
+  g_clear_pointer (&data->marker_types, g_free);
+  g_slice_free (InkLevelData, data);
+}
+
 static void
 pp_printer_entry_get_property (GObject    *object,
                                guint       prop_id,
@@ -160,6 +176,7 @@ static void
 pp_printer_entry_init (PpPrinterEntry *self)
 {
   gtk_widget_init_template (GTK_WIDGET (self));
+  self->inklevel = ink_level_data_new ();
 }
 
 typedef struct {
@@ -226,7 +243,7 @@ sanitize_printer_model (gchar *printer_make_and_model)
 
   g_free (tmp);
 
-  return g_strdup (printer_model);
+  return printer_model;
 }
 
 static gboolean
@@ -623,11 +640,8 @@ pp_printer_entry_update_jobs_count (PpPrinterEntry *self)
 {
   PpPrinter *printer;
 
-  if (self->get_jobs_cancellable != NULL)
-    {
-      g_cancellable_cancel (self->get_jobs_cancellable);
-      g_clear_object (&self->get_jobs_cancellable);
-    }
+  g_cancellable_cancel (self->get_jobs_cancellable);
+  g_clear_object (&self->get_jobs_cancellable);
 
   self->get_jobs_cancellable = g_cancellable_new ();
 
@@ -662,17 +676,31 @@ printer_jobs_dialog_free_cb (GtkDialog *dialog,
   pp_jobs_dialog_free ((PpJobsDialog *) user_data);
 }
 
+void
+pp_printer_entry_show_jobs_dialog (PpPrinterEntry *self)
+{
+  if (self->pp_jobs_dialog == NULL)
+    {
+      self->pp_jobs_dialog = pp_jobs_dialog_new (
+        GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))),
+        jobs_dialog_response_cb,
+        self,
+        self->printer_name);
+    }
+}
+
+void
+pp_printer_entry_authenticate_jobs (PpPrinterEntry *self)
+{
+  pp_printer_entry_show_jobs_dialog (self);
+  pp_jobs_dialog_authenticate_jobs (self->pp_jobs_dialog);
+}
+
 static void
 show_jobs_dialog (GtkButton *button,
                   gpointer   user_data)
 {
-  PpPrinterEntry *self = PP_PRINTER_ENTRY (user_data);
-
-  self->pp_jobs_dialog = pp_jobs_dialog_new (
-    GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (self))),
-    jobs_dialog_response_cb,
-    self,
-    self->printer_name);
+  pp_printer_entry_show_jobs_dialog (PP_PRINTER_ENTRY (user_data));
 }
 
 enum
@@ -786,8 +814,6 @@ pp_printer_entry_new (cups_dest_t  printer,
 
   self = g_object_new (PP_PRINTER_ENTRY_TYPE, "printer-name", printer.name, NULL);
 
-  self->inklevel = g_slice_new0 (InkLevelData);
-
   if (printer.instance)
     {
       instance = g_strdup_printf ("%s / %s", printer.name, printer.instance);
@@ -994,18 +1020,14 @@ pp_printer_entry_dispose (GObject *object)
   g_clear_pointer (&self->printer_location, g_free);
   g_clear_pointer (&self->printer_make_and_model, g_free);
   g_clear_pointer (&self->printer_hostname, g_free);
+  g_clear_pointer (&self->inklevel, ink_level_data_free);
 
-  if (self->get_jobs_cancellable != NULL)
-    {
-      g_cancellable_cancel (self->get_jobs_cancellable);
-      g_clear_object (&self->get_jobs_cancellable);
-    }
+  g_cancellable_cancel (self->get_jobs_cancellable);
+  g_clear_object (&self->get_jobs_cancellable);
+
+  g_cancellable_cancel (self->check_clean_heads_cancellable);
+  g_clear_object (&self->check_clean_heads_cancellable);
 
-  if (self->check_clean_heads_cancellable)
-    {
-      g_cancellable_cancel (self->check_clean_heads_cancellable);
-      g_clear_object (&self->check_clean_heads_cancellable);
-    }
   g_clear_object (&self->clean_command);
 
   G_OBJECT_CLASS (pp_printer_entry_parent_class)->dispose (object);
