@@ -24,6 +24,7 @@
 
 #include <config.h>
 
+#include "cc-debug.h"
 #include "cc-window.h"
 
 #include <glib/gi18n.h>
@@ -35,9 +36,9 @@
 #include <string.h>
 #include <time.h>
 
+#include "cc-application.h"
 #include "cc-panel.h"
 #include "cc-shell.h"
-#include "cc-shell-category-view.h"
 #include "cc-shell-model.h"
 #include "cc-panel-list.h"
 #include "cc-panel-loader.h"
@@ -64,7 +65,6 @@ struct _CcWindow
   GtkWidget  *search_bar;
   GtkWidget  *search_entry;
   GtkWidget  *lock_button;
-  GtkWidget  *current_panel_box;
   GtkWidget  *development_warning_dialog;
   GtkWidget  *current_panel;
   char       *current_panel_id;
@@ -88,7 +88,8 @@ G_DEFINE_TYPE_WITH_CODE (CcWindow, cc_window, GTK_TYPE_APPLICATION_WINDOW,
 enum
 {
   PROP_0,
-  PROP_ACTIVE_PANEL
+  PROP_ACTIVE_PANEL,
+  PROP_MODEL
 };
 
 /* Auxiliary methods */
@@ -158,20 +159,46 @@ get_symbolic_icon_name_from_g_icon (GIcon *gicon)
   return NULL;
 }
 
+static void
+remove_all_custom_widgets (CcWindow *self)
+{
+  GtkWidget *widget;
+  guint i;
+
+  CC_ENTRY;
+
+  /* remove from the header */
+  for (i = 0; i < self->custom_widgets->len; i++)
+    {
+      widget = g_ptr_array_index (self->custom_widgets, i);
+      gtk_container_remove (GTK_CONTAINER (self->top_right_box), widget);
+    }
+  g_ptr_array_set_size (self->custom_widgets, 0);
+
+  CC_EXIT;
+}
+
 static gboolean
-activate_panel (CcWindow    *self,
-                const gchar *id,
-                GVariant    *parameters,
-                const gchar *name,
-                GIcon       *gicon)
+activate_panel (CcWindow          *self,
+                const gchar       *id,
+                GVariant          *parameters,
+                const gchar       *name,
+                GIcon             *gicon,
+                CcPanelVisibility  visibility)
 {
   g_autoptr (GTimer) timer = NULL;
-  GtkWidget *box, *title_widget;
+  GtkWidget *title_widget;
   gdouble ellapsed_time;
 
   if (!id)
     return FALSE;
 
+  if (visibility == CC_PANEL_HIDDEN)
+    return FALSE;
+
+  /* clear any custom widgets */
+  remove_all_custom_widgets (self);
+
   timer = g_timer_new ();
 
   g_settings_set_string (self->settings, "last-panel", id);
@@ -186,15 +213,10 @@ activate_panel (CcWindow    *self,
   gtk_lock_button_set_permission (GTK_LOCK_BUTTON (self->lock_button),
                                   cc_panel_get_permission (CC_PANEL (self->current_panel)));
 
-  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-
-  gtk_box_pack_start (GTK_BOX (box), self->current_panel,
-                      TRUE, TRUE, 0);
-
-  gtk_stack_add_named (GTK_STACK (self->stack), box, id);
+  gtk_stack_add_named (GTK_STACK (self->stack), self->current_panel, id);
 
   /* switch to the new panel */
-  gtk_widget_show (box);
+  gtk_widget_show (self->current_panel);
   gtk_stack_set_visible_child_name (GTK_STACK (self->stack), id);
 
   /* set the title of the window */
@@ -204,8 +226,6 @@ activate_panel (CcWindow    *self,
   title_widget = cc_panel_get_title_widget (CC_PANEL (self->current_panel));
   gtk_header_bar_set_custom_title (GTK_HEADER_BAR (self->panel_headerbar), title_widget);
 
-  self->current_panel_box = box;
-
   /* Finish profiling */
   g_timer_stop (timer);
 
@@ -216,21 +236,6 @@ activate_panel (CcWindow    *self,
   return TRUE;
 }
 
-static void
-remove_all_custom_widgets (CcWindow *self)
-{
-  GtkWidget *widget;
-  guint i;
-
-  /* remove from the header */
-  for (i = 0; i < self->custom_widgets->len; i++)
-    {
-      widget = g_ptr_array_index (self->custom_widgets, i);
-      gtk_container_remove (GTK_CONTAINER (self->top_right_box), widget);
-    }
-  g_ptr_array_set_size (self->custom_widgets, 0);
-}
-
 static void
 add_current_panel_to_history (CcShell    *shell,
                               const char *start_id)
@@ -248,14 +253,6 @@ add_current_panel_to_history (CcShell    *shell,
   g_debug ("Added '%s' to the previous panels", self->current_panel_id);
 }
 
-static void
-shell_show_overview_page (CcWindow *self)
-{
-  /* TODO: need design input on a possibly new overview page. For now,
-   * just go back to the main section of the panel list. */
-  cc_panel_list_set_view (CC_PANEL_LIST (self->panel_list), CC_PANEL_LIST_MAIN);
-}
-
 static void
 update_list_title (CcWindow *self)
 {
@@ -288,6 +285,23 @@ update_list_title (CcWindow *self)
     gtk_header_bar_set_title (GTK_HEADER_BAR (self->header), title);
 }
 
+static void
+on_row_changed_cb (GtkTreeModel *model,
+                   GtkTreePath  *path,
+                   GtkTreeIter  *iter,
+                   CcWindow     *self)
+{
+  g_autofree gchar *id = NULL;
+  CcPanelVisibility visibility;
+
+  gtk_tree_model_get (model, iter,
+                      COL_ID, &id,
+                      COL_VISIBILITY, &visibility,
+                      -1);
+
+  cc_panel_list_set_panel_visibility (CC_PANEL_LIST (self->panel_list), id, visibility);
+}
+
 static void
 setup_model (CcWindow *shell)
 {
@@ -295,7 +309,9 @@ setup_model (CcWindow *shell)
   GtkTreeIter iter;
   gboolean valid;
 
-  shell->store = (GtkListStore *) cc_shell_model_new ();
+  /* CcApplication must have a valid model at this point */
+  g_assert (shell->store != NULL);
+
   model = GTK_TREE_MODEL (shell->store);
 
   cc_panel_loader_fill_model (CC_SHELL_MODEL (shell->store));
@@ -312,6 +328,7 @@ setup_model (CcWindow *shell)
       g_autofree gchar *id = NULL;
       g_autofree gchar *icon_name = NULL;
       g_autofree GStrv keywords = NULL;
+      CcPanelVisibility visibility;
 
       gtk_tree_model_get (model, &iter,
                           COL_CATEGORY, &category,
@@ -320,6 +337,7 @@ setup_model (CcWindow *shell)
                           COL_ID, &id,
                           COL_NAME, &name,
                           COL_KEYWORDS, &keywords,
+                          COL_VISIBILITY, &visibility,
                           -1);
 
       icon_name = get_symbolic_icon_name_from_g_icon (icon);
@@ -330,10 +348,14 @@ setup_model (CcWindow *shell)
                                name,
                                description,
                                keywords,
-                               icon_name);
+                               icon_name,
+                               visibility);
 
       valid = gtk_tree_model_iter_next (model, &iter);
     }
+
+  /* React to visibility changes */
+  g_signal_connect_object (model, "row-changed", G_CALLBACK (on_row_changed_cb), shell, 0);
 }
 
 
@@ -341,10 +363,12 @@ static gboolean
 set_active_panel_from_id (CcShell      *shell,
                           const gchar  *start_id,
                           GVariant     *parameters,
+                          gboolean      add_to_history,
                           GError      **error)
 {
   g_autoptr(GIcon) gicon = NULL;
   g_autofree gchar *name = NULL;
+  CcPanelVisibility visibility;
   GtkTreeIter iter;
   GtkWidget *old_panel;
   CcWindow *self;
@@ -360,9 +384,6 @@ set_active_panel_from_id (CcShell      *shell,
       return TRUE;
     }
 
-  /* clear any custom widgets */
-  remove_all_custom_widgets (self);
-
   iter_valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (self->store), &iter);
 
   /* find the details for this item */
@@ -374,6 +395,7 @@ set_active_panel_from_id (CcShell      *shell,
                           COL_NAME, &name,
                           COL_GICON, &gicon,
                           COL_ID, &id,
+                          COL_VISIBILITY, &visibility,
                           -1);
 
       if (id && strcmp (id, start_id) == 0)
@@ -385,7 +407,7 @@ set_active_panel_from_id (CcShell      *shell,
       iter_valid = gtk_tree_model_iter_next (GTK_TREE_MODEL (self->store), &iter);
     }
 
-  old_panel = self->current_panel_box;
+  old_panel = self->current_panel;
 
   if (!name)
     {
@@ -394,14 +416,19 @@ set_active_panel_from_id (CcShell      *shell,
     }
 
   /* Activate the panel */
-  activated = activate_panel (CC_WINDOW (shell), start_id, parameters, name, gicon);
+  activated = activate_panel (CC_WINDOW (shell), start_id, parameters, name, gicon, visibility);
 
   /* Failed to activate the panel for some reason, let's keep the old
    * panel around instead */
   if (!activated)
-    return TRUE;
+    {
+      g_debug ("Failed to activate panel");
+      return TRUE;
+    }
+
+  if (add_to_history)
+    add_current_panel_to_history (shell, start_id);
 
-  /* Successful activation */
   g_free (self->current_panel_id);
   self->current_panel_id = g_strdup (start_id);
 
@@ -428,23 +455,40 @@ set_active_panel (CcWindow *shell,
       /* set the new panel */
       if (panel)
         shell->active_panel = g_object_ref (panel);
-      else
-        shell_show_overview_page (shell);
 
       g_object_notify (G_OBJECT (shell), "active-panel");
     }
 }
 
+static void
+switch_to_previous_panel (CcWindow *self)
+{
+  g_autofree gchar *previous_panel_id = NULL;
+
+  CC_ENTRY;
+
+  if (g_queue_get_length (self->previous_panels) == 0)
+    CC_RETURN ();
+
+  previous_panel_id = g_queue_pop_head (self->previous_panels);
+
+  g_debug ("Going to previous panel (%s)", previous_panel_id);
+
+  set_active_panel_from_id (CC_SHELL (self), previous_panel_id, NULL, FALSE, NULL);
+
+  CC_EXIT;
+}
+
 /* Callbacks */
 static void
 show_panel_cb (CcPanelList *panel_list,
                const gchar *panel_id,
                CcWindow    *self)
 {
-  if (panel_id)
-    set_active_panel_from_id (CC_SHELL (self), panel_id, NULL, NULL);
-  else
-    shell_show_overview_page (self);
+  if (!panel_id)
+    return;
+
+  set_active_panel_from_id (CC_SHELL (self), panel_id, NULL, TRUE, NULL);
 }
 
 static void
@@ -519,17 +563,6 @@ window_map_event_cb (GtkWidget *widget,
   return GDK_EVENT_PROPAGATE;
 }
 
-static gboolean
-window_button_release_event_cb (GtkWidget      *win,
-                                GdkEventButton *event,
-                                CcWindow       *self)
-{
-  /* back button */
-  if (event->button == MOUSE_BACK_BUTTON)
-    shell_show_overview_page (self);
-  return FALSE;
-}
-
 static gboolean
 window_key_press_event_cb (GtkWidget   *win,
                            GdkEventKey *event,
@@ -591,7 +624,8 @@ window_key_press_event_cb (GtkWidget   *win,
            (is_rtl && state == GDK_MOD1_MASK && event->keyval == GDK_KEY_Right) ||
            event->keyval == GDK_KEY_Back)
     {
-      previous_button_clicked_cb (NULL, self);
+      g_debug ("Going to previous panel");
+      switch_to_previous_panel (self);
       retval = GDK_EVENT_STOP;
     }
 
@@ -640,8 +674,7 @@ cc_window_set_active_panel_from_id (CcShell      *shell,
                                     GVariant     *parameters,
                                     GError      **error)
 {
-  add_current_panel_to_history (shell, start_id);
-  return set_active_panel_from_id (shell, start_id, parameters, error);
+  return set_active_panel_from_id (shell, start_id, parameters, TRUE, error);
 }
 
 static void
@@ -650,11 +683,15 @@ cc_window_embed_widget_in_header (CcShell   *shell,
 {
   CcWindow *self = CC_WINDOW (shell);
 
+  CC_ENTRY;
+
   /* add to header */
   gtk_box_pack_end (GTK_BOX (self->top_right_box), widget, FALSE, FALSE, 0);
   g_ptr_array_add (self->custom_widgets, g_object_ref (widget));
 
   gtk_size_group_add_widget (self->header_sizegroup, widget);
+
+  CC_EXIT;
 }
 
 static GtkWidget *
@@ -698,6 +735,11 @@ cc_window_get_property (GObject    *object,
     case PROP_ACTIVE_PANEL:
       g_value_set_object (value, self->active_panel);
       break;
+
+    case PROP_MODEL:
+      g_value_set_object (value, self->store);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -716,11 +758,49 @@ cc_window_set_property (GObject      *object,
     case PROP_ACTIVE_PANEL:
       set_active_panel (shell, g_value_get_object (value));
       break;
+
+    case PROP_MODEL:
+      g_assert (shell->store == NULL);
+      shell->store = g_value_dup_object (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
 }
 
+static void
+cc_window_constructed (GObject *object)
+{
+  g_autofree char *id = NULL;
+  GtkSettings *settings;
+  CcWindow *self;
+
+  self = CC_WINDOW (object);
+
+  /* Handle decorations for the split headers. */
+  settings = gtk_settings_get_default ();
+  g_signal_connect (settings,
+                    "notify::gtk-decoration-layout",
+                    G_CALLBACK (split_decorations_cb),
+                    self);
+
+  split_decorations_cb (settings, NULL, self);
+
+  /* Add the panels */
+  setup_model (self);
+
+  /* After everything is loaded, select the last used panel, if any,
+   * or the first visible panel */
+  id = g_settings_get_string (self->settings, "last-panel");
+  if (id != NULL && cc_shell_model_has_panel (CC_SHELL_MODEL (self->store), id))
+    cc_panel_list_set_active_panel (CC_PANEL_LIST (self->panel_list), id);
+  else
+    cc_panel_list_activate (CC_PANEL_LIST (self->panel_list));
+
+  G_OBJECT_CLASS (cc_window_parent_class)->constructed (object);
+}
+
 static void
 cc_window_dispose (GObject *object)
 {
@@ -758,6 +838,7 @@ cc_window_class_init (CcWindowClass *klass)
 
   object_class->get_property = cc_window_get_property;
   object_class->set_property = cc_window_set_property;
+  object_class->constructed = cc_window_constructed;
   object_class->dispose = cc_window_dispose;
   object_class->finalize = cc_window_finalize;
 
@@ -765,6 +846,14 @@ cc_window_class_init (CcWindowClass *klass)
 
   g_object_class_override_property (object_class, PROP_ACTIVE_PANEL, "active-panel");
 
+  g_object_class_install_property (object_class,
+                                   PROP_MODEL,
+                                   g_param_spec_object ("model",
+                                                        "Model",
+                                                        "The CcShellModel of this application",
+                                                        CC_TYPE_SHELL_MODEL,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/ControlCenter/gtk/window.ui");
 
   gtk_widget_class_bind_template_child (widget_class, CcWindow, development_warning_dialog);
@@ -789,7 +878,6 @@ cc_window_class_init (CcWindowClass *klass)
   gtk_widget_class_bind_template_callback (widget_class, search_entry_activate_cb);
   gtk_widget_class_bind_template_callback (widget_class, show_panel_cb);
   gtk_widget_class_bind_template_callback (widget_class, update_list_title);
-  gtk_widget_class_bind_template_callback (widget_class, window_button_release_event_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_key_press_event_cb);
   gtk_widget_class_bind_template_callback (widget_class, window_map_event_cb);
 
@@ -799,45 +887,22 @@ cc_window_class_init (CcWindowClass *klass)
 static void
 cc_window_init (CcWindow *self)
 {
-  GtkSettings *settings;
-  g_autofree char *id = NULL;
-
   gtk_widget_init_template (GTK_WIDGET (self));
 
   gtk_widget_add_events (GTK_WIDGET (self), GDK_BUTTON_RELEASE_MASK);
 
   self->settings = g_settings_new ("org.gnome.ControlCenter");
-
-  /* Handle decorations for the split headers. */
-  settings = gtk_settings_get_default ();
-  g_signal_connect (settings,
-                    "notify::gtk-decoration-layout",
-                    G_CALLBACK (split_decorations_cb),
-                    self);
-
-  split_decorations_cb (settings, NULL, self);
-
-  /* Add the panels */
   self->custom_widgets = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
   self->previous_panels = g_queue_new ();
 
-  setup_model (self);
-
-  /* After everything is loaded, select the last used panel, if any,
-   * or the first visible panel */
-  id = g_settings_get_string (self->settings, "last-panel");
-  if (id != NULL && cc_shell_model_has_panel (CC_SHELL_MODEL (self->store), id))
-    cc_panel_list_set_active_panel (CC_PANEL_LIST (self->panel_list), id);
-  else
-    cc_panel_list_activate (CC_PANEL_LIST (self->panel_list));
-
   /* Add a custom CSS class on development builds */
   if (in_flatpak_sandbox ())
     add_development_build_css (self);
 }
 
 CcWindow *
-cc_window_new (GtkApplication *application)
+cc_window_new (GtkApplication *application,
+               CcShellModel   *model)
 {
   g_return_val_if_fail (GTK_IS_APPLICATION (application), NULL);
 
@@ -848,20 +913,14 @@ cc_window_new (GtkApplication *application)
                        "icon-name", DEFAULT_WINDOW_ICON_NAME,
                        "window-position", GTK_WIN_POS_CENTER,
                        "show-menubar", FALSE,
+                       "model", model,
                        NULL);
 }
 
-void
-cc_window_set_overview_page (CcWindow *center)
-{
-  shell_show_overview_page (center);
-}
-
 void
 cc_window_set_search_item (CcWindow   *center,
                            const char *search)
 {
-  shell_show_overview_page (center);
   gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (center->search_bar), TRUE);
   gtk_entry_set_text (GTK_ENTRY (center->search_entry), search);
   gtk_editable_set_position (GTK_EDITABLE (center->search_entry), -1);
